<?php
/*--------------------------------------------+
| GameStatusViewer     V 1.0     2005-07-26   |
| Source Engine Protocoll                            |
|---------------------------------------------|
| Copyright Makro (Markus Kronier)  2005     |
| web: http://www.makromobil.de               |
| email: makro@makromobil.de                  |
|---------------------------------------------|
| Released under the terms and conditions of  |
| the GNU General Public License (Version 2)  |
+--------------------------------------------*/

class CProt_SourceEngine {

  var $m_serverIp;
  var $m_queryPort;

  //---------------
  var $m_requestCmd          = array();
  //---------------

  var $m_serverData          = array();

  function CProt_SourceEngine($serverIp, $queryPort) {
    $this->m_serverIp        = $serverIp;
    $this->m_queryPort       = $queryPort;
    $this->m_requestCmd[]    = "\xFF\xFF\xFF\xFF\x54";  //--- Details
    $this->m_requestCmd[]    = "\xFF\xFF\xFF\xFF\x56";  //--- Rules
    $this->m_requestCmd[]    = "\xFF\xFF\xFF\xFF\x55";  //--- Players
  }

  function ReadServerData() {
    $error = "";
    $this->m_serverData = array();

    $socket = @fsockopen('udp://'. $this->m_serverIp, $this->m_queryPort, $errno, $errstr, 10);
    if ($socket) {
      socket_set_timeout($socket, 2);
      foreach ($this->m_requestCmd as $reqCmd) {
        //--- Send request to Server
        fwrite($socket, $reqCmd);

        //--- Read response
        $part = fread($socket,2048);
        if ($part) $this->m_serverData[] = $part;
      }
      fclose($socket);
    } else {
      $error = "not connected to the server!";
    }
    return $error;
  }

  function GetGameData() {
    $data     = substr($this->m_serverData[0], 6);
    $data     = explode("\x00",$data,5);

    $gameData = array();

    $gameData["hostip"] = $this->m_serverIp;
    $gameData["queryport"] = $this->m_queryPort;

    $gameData["hostname"] = $data[0];
    $gameData["mapname"]  = $data[1];
    $gameData["gamename"] = $data[3];

    $values = unpack("C*",$data[4]);

    $gameData["numplayers"] = $values[3];
    $gameData["maxplayers"] = $values[4];
    $gameData["numbots"]   = $values[5];

    if (chr($values[6]) == "d") {
      $gameData["dedicated"] = "1";
    } else {
      $gameData["dedicated"] = "0";
    }
    if (chr($values[7]) == "l") {$gameData["serveros"] = "Linux";}
    if (chr($values[7]) == "w") {$gameData["serveros"] = "Windows";}

    return $gameData;
  }

  function GetPlayerData() {
    $playerData = array();
    if (substr($this->m_serverData[2],0,5)=="\xFF\xFF\xFF\xFF\x44") {
      $data = substr($this->m_serverData[2], 6);
      $noffs = 0;
      $nPl  = 0;
      $nPlCount = ord($this->m_serverData[2][5]);
      $nlenData = strlen($data);

      while (($nPl < $nPlCount) && ($noffs < $nlenData)) {
        //--- PlayerNr
        $nPlId = ord($data[$noffs++]);

        //--- Playername
        $sPlayer = "";
        do {
          $sPlayer .= $data[$noffs++];
        } while (($data[$noffs] != "\x00") && ($noffs < $nlenData));
        $noffs++;
        //-- Score + time
        $bdata = unpack("lscore/ftime", substr($data,$noffs,8));

        $min = floor($bdata["time"] / 60);
		$h   = floor($min / 60);
		$sec = floor($bdata["time"] - ($min * 60));
		$min = $min - ($h * 60);
        $bdata["time"] = sprintf("%02s:%02s:%02s", $h, $min, $sec);

        $noffs += 8;

        if ($noffs <=  $nlenData) {
          $playerData["id_".$nPl] = $nPl+1;
          $playerData["player_".$nPl] = $sPlayer;
          $playerData["score_".$nPl] = $bdata["score"];
          $playerData["time_".$nPl] = $bdata["time"];
        }
        $nPl++;
      }
    }
    return $playerData;
  }
}

?>